#ifndef __BITSET_H_
#define __BITSET_H_

#include <glib.h>
#include <stdio.h>
#include <stdlib.h>


/* This data type is a set of max_id-1 elements. Each element is an bs_element 
   (an unsigned integer). For each element we set to 1 a bit in a chain of bits
   (so for 'n' elements from 0 to 1 we have 'n+1 bits'. 0 is always the first 
   element */

typedef guint32 bs_element;

typedef struct bitset_struct {
	GArray *array;
	bs_element max_id;          /* maximum value of elements in the set */
	bs_element it;				/* iterator */
	bs_element total;           /* total number of elements in the set */
} *BitSet;


/* Creates a new bitset which contains elements up to 'max_id' */
BitSet bitset_new(bs_element max_id);

/* Creates a new bitset as result of the intersection of bs_a and bs_b sets */
BitSet bitset_new_intersection(BitSet bs_a, BitSet bs_b);

/* Creates a new bitset as copy of another */
BitSet bitset_new_copy(BitSet bs_a);

/* Creates a new bitset as result of the union of bs_a and bs_b sets */
BitSet bitset_new_union(BitSet bs_a, BitSet bs_b);

/* Creates a new bitset as the difference between two */
BitSet bitset_new_diff(BitSet bs_a, BitSet bs_b);

/* Fills the set with all possible elements (all bits to 1) */
void bitset_fullfill (BitSet *bs);

void bitset_free(BitSet *bs);

/* Returns TRUE if 'id' is an element of 'bs', FALSE otherwise */
gboolean bitset_in(BitSet bs, bs_element id);

/* Adds elements of the given set */
void bitset_add_set(BitSet *bs, BitSet bs_a);

/* 	Adds a new element in the set. If the element was already there, 
 	there is no change */
void bitset_insert(BitSet *bs, bs_element id);

/* Removes element from the set if it was there */
void bitset_delete(BitSet *bs, bs_element id);

/* Prints to standard output elements in the set */
void bitset_dump(BitSet bs);

/* Stores the bitset in a file */
void bitset_serialize(BitSet bs, FILE *f);

/* Reads the bitset from a file at the given position */
BitSet bitset_new_deserialize(FILE *f, long offset);

/* Iteration operations */
void bitset_it_reset(BitSet *bs);
bs_element bitset_it_next(BitSet *bs);
gboolean bitset_it_end(BitSet bs);


#endif /*__BITSET_H_*/
