/***************************************************************************
 *            class_info.c
 *
 *  Thu Nov 27 12:18:10 2003
 *  Copyright  2003  Arturo Montejo Raez
 *  arturo.montejo.raez@cern.ch
 ****************************************************************************/

#include <stdio.h>
#include "class_info.h"

/* ---------------------------------------------------------------------------
   PUBLIC Functions
   --------------------------------------- */

/* Compute document frequency for each class */
ClassInfoArray
class_info_new_compute(InvertedIndex class_doc, BitSet train_docs, BitSet eval_docs, BitSet test_docs)
{
  tc_id i;
  BitSet class_docs, train_class_docs, eval_class_docs, test_class_docs;

  /* We create the array */
  GArray *class_info = g_array_new (FALSE, TRUE, sizeof (ClassInfo));
  class_info = g_array_set_size (class_info, class_doc->max_id + 1);
	
  tc_message ("\tComputing class info...");

  /* compute class info */
  for (i = 0; i <= class_doc->max_id; i++)
    {
      ClassInfo *cinf = &g_array_index (class_info, ClassInfo, i);
      cinf->id = cinf->new_id = i;
      cinf->method = NULL;

      class_docs = inverted_index_get (class_doc, i);
	  
      train_class_docs =
		bitset_new_intersection (train_docs, class_docs);
      cinf->train_df = train_class_docs->total;
      bitset_free (&train_class_docs);
	  
      eval_class_docs =
		bitset_new_intersection (eval_docs, class_docs);
      cinf->eval_df = eval_class_docs->total;
      bitset_free (&eval_class_docs);
	  
      test_class_docs =
		bitset_new_intersection (test_docs, class_docs);
      cinf->test_df = test_class_docs->total;
      bitset_free (&test_class_docs);
	  
      bitset_free (&class_docs);
    }
  tc_message ("done\n");
  return class_info;
}

/* -------------------------------------------------------------------------- */

/* This function sets to -1 those classes which should be
   discarded. When a class is kept, the new id is just a number with
   no practical utility (only to be different to -1). A classes is
   represented by the id value set when the model was constructed and
   never changed (as opossite to features, which do change their ids
   and no longer can be compared to those of the model). */
void
class_info_reduce(ClassInfoArray *class_info, guint df_min)
{
  tc_id i, new_id = 0;
  ClassInfo *cinf;

  tc_message ("\tReducing classes...");

  /* filter by document frequency and when not appearing on three sets */
  if (df_min > 0) {
	for (i = 0; i < (*class_info)->len; i++) {
	  cinf = &g_array_index (*class_info, ClassInfo, i);
	  if (cinf->train_df < df_min || cinf->eval_df == 0 || cinf->test_df == 0)
	    cinf->new_id = -1;
	}
  }

  /* sets new ids */	
  for (i = 0; i < (*class_info)->len; i++) {
	cinf =	&g_array_index (*class_info, ClassInfo, i);
	if (cinf->new_id != -1)
	  cinf->new_id = new_id++;
  }
  tc_message_int ("done (reduced to %d classes)\n", new_id);
}
/* -------------------------------------------------------------------------- */
/* Saves to disk */
void
class_info_save(ClassInfoArray class_info, gchar *filepath)
{
  FILE *fhnd;
  if (!(fhnd = fopen(filepath, "wb"))) 
    tc_error("class_info_save: Unable to create file\n"); 
  fwrite(&class_info->len, sizeof(guint), 1, fhnd); 
  fwrite(class_info->data, sizeof(ClassInfo), class_info->len, fhnd); 
  fclose(fhnd);
}

/* -------------------------------------------------------------------------- */
/* Reads from disk */
ClassInfoArray
class_info_new_read(gchar *filepath)
{	
  ClassInfoArray class_info = NULL;
  FILE *fhnd;
  guint len; 

  if (!(fhnd = fopen(filepath, "rb"))) 
    tc_error("class_info_new_read: Unable to open info file\n"); 
  class_info = g_array_new(FALSE, TRUE, sizeof(ClassInfo)); 
  fread(&len, sizeof(guint), 1, fhnd); 
  class_info = g_array_set_size(class_info, len); 
  if (!class_info)
    tc_error("class_info_new_read: No memory for class_info");
  fread(class_info->data, sizeof(ClassInfo), len, fhnd); 
  fclose(fhnd);
  return class_info;	
}
/* -------------------------------------------------------------------------- */
gint __class_info_cmp_by_df(gconstpointer a, gconstpointer b) {
  ClassInfo *acinf = (ClassInfo *)b, *bcinf = (ClassInfo *)a;
  return (acinf->train_df+acinf->eval_df+acinf->test_df) - 
    (bcinf->train_df+bcinf->eval_df+bcinf->test_df);
}
/* -------------------------------------------------------------------------- */
void class_info_sort_by_df(ClassInfoArray *class_info) {
  g_array_sort(*class_info, __class_info_cmp_by_df);
}
/* -------------------------------------------------------------------------- */
ClassInfo* class_info_get(ClassInfoArray class_info, guint index)
{
  return &g_array_index(class_info, ClassInfo, index);
}
/* -------------------------------------------------------------------------- */
/* Frees memory */
void
class_info_free(ClassInfoArray *class_info)
{
  g_array_free(*class_info, TRUE);
}
