/***************************************************************************
 *            class_info.h
 *
 *  Thu Nov 27 12:14:20 2003
 *  Copyright  2003  Arturo Montejo Raez
 *  arturo.montejo.raez@cern.ch
 ****************************************************************************/

#ifndef _CLASS_INFO_H
#define _CLASS_INFO_H

#include <glib.h>
#include "tc_utils.h"
#include "inverted_index.h"
#include "bitset.h"
#include "method.h"

/* ---------------------------------------------------------------------------
   Data definitions and macros
   --------------------------------------- */

/* This structure stores global information about each class */
typedef struct class_info_struct
{
	tc_id id;       /* id of the class */
	tc_id new_id;	/* id after reduction (-1 means discarded feature) */
	guint train_df;		/* number of documents the class is attached to */
	guint eval_df;		/* number of documents the class is attached to */
	guint test_df;		/* number of documents the class is attached to */
	Method method; /* pointer to the method selected for this class in training */
	Method filter; /* pointer to the method selected for this class in training for filtering */
} ClassInfo;

typedef GArray *ClassInfoArray;

/* ---------------------------------------------------------------------------
   Functions
   --------------------------------------- */

/* Compute document frequency for each class */
ClassInfoArray
class_info_new_compute(InvertedIndex class_doc, BitSet train_docs, BitSet eval_docs, BitSet test_docs);

/* This function sets to -1 those classes which should be discarded */
void
class_info_reduce(ClassInfoArray *class_info, guint df_min);

/* Saves to disk */
void
class_info_save(ClassInfoArray class_info, gchar *filepath);

/* Reads from disk */
ClassInfoArray
class_info_new_read(gchar *filepath);

/* Frees memory */
void
class_info_free(ClassInfoArray *class_info);

/* Gets a value */
ClassInfo *
class_info_get(ClassInfoArray class_info, guint index);

/* Sorts by document frequency */
void class_info_sort_by_df(ClassInfoArray *class_info);

#endif /* _CLASS_INFO_H */
