/* 
   Learning Vector Quantization algorithm
   Implementation of LVQ1 (see http://www.cis.hut.fi/~hynde/lvq/)
*/

#ifndef __LVQ_H
#define __LVQ_H

#include <glib.h>
#include "samples.h"
#include "weights.h"

typedef struct lvq_struct
{
  guint iterations;      /* number of runs of the LVQ loop */
  double alpha_init;     /* initial learning rate */
  double alpha_threshold; /* threshold for learning rate */
  

  gchar init;            /* initialization method for codebook vectors KNN o Means */
  guint npos_cbv;        /* number of positive codebook vectors */
  guint nneg_cbv;        /* number of negative codebook vectors */
  Weights *pos_cbv;      /* positive codebook vectors */
  Weights *neg_cbv;      /* negative codebook vectors */
  guint normcbv;         /* normalization base for balancing codebook vectors */
  
} *LVQ;


void *lvq_new(gchar *args);
void lvq_free(void **cp);
void lvq_save(void *cp, gchar *file);
void *lvq_new_read(gchar *file);
double lvq_classify(void *cp, Sample s);
void lvq_dump (void* cp);
void lvq_learn(void **cp, Samples ss);

#endif /* __LVQ_H */
