/*---------------------------------------------------------------------
  ML-Tree

  A classification tree for multi-label, multi-level and 
  multi-source data.

  This tree builds a node per each class selecting that one with the
  highest score of balance.

---------------------------------------------------------------------*/

#ifndef _ML_TREE_H_
#define _ML_TREE_H_

#include <glib.h>
#include "method.h"
#include "fold.h"

typedef Method MLNodeInfo;
typedef GNode MLNode;

MLNode *
mltree_new(Fold *f, BitSet classes, 
	   BitSet train_docs, BitSet eval_docs, 
	   Samples train_samples, Samples eval_samples,
	   BitSet *mclass_docs, 
	   FoldTrainParams *params, guint depth);

/* 
   Classifies a document. 
   ntop = 0 means select all positive classes 
*/
BitSet mltree_classify(MLNode *node, Sample s, Fold f, guint ntop, FoldTrainParams *params);

void mltree_free(MLNode *node);
void mltree_save(MLNode *node, gchar *filename);
MLNode *mltree_new_read(gchar *filename);
void mltree_dump(MLNode *node);


#endif /*_ML_TREE_H_*/
