#ifndef __WEIGHTS_H_
#define __WEIGHTS_H_

#include <glib.h>
#include <stdlib.h>
#include <stdio.h>
#include "samples.h"
#include "intstr_table.h"

/* ---------------------------------------------------------------------
   Weights type
 */

typedef GArray *Weights;	/* An array of float values */

Weights 
weights_new(guint max_index);

void 
weights_resize(Weights *w, guint max_index);

Weights 
weights_new_from_sample(Sample s);

Weights 
weights_clone(Weights w);

void 
weights_free (Weights * w);

void 
weights_dump (Weights w);

inline void 
weights_set(Weights *w, guint index, double value);

void
weights_set_all(Weights *w, double value);

inline double 
weights_get(Weights w, guint index);

void 
weights_add_weight(Weights *w, guint index, double value);

double 
weights_product_by_sample(Weights w, Sample s);

void 
weights_product_weights(Weights *w, Weights w2);

void 
weights_divide_weights(Weights *w, Weights w2);

void 
weights_add_weights(Weights *w, Weights w2);

void 
weights_product_by_scalar(Weights *w, double v);

double
weights_distance_to_sample(Weights w, Sample s);

void 
weights_save(Weights w, FILE *f);

void 
weights_save_ascii(Weights w, FILE *f, IntStrTable item_str, GArray *idid);

Weights 
weights_new_read(FILE *f);

/* Computes threshold as the middle point of average of classifying
   values for positive and negative samples */

double 
weights_threshold(Weights w, Samples ss);

#endif /*__WEIGHTS_H_*/
