#ifndef __FEATURE_H_
#define __FEATURE_H_

#include <glib.h>
#include "tc_types.h"

/* ---------------------------------------------------------------------
   Feature type
 */
typedef struct feature_struct
{
  tc_id id;				/* term identificator */
  double w;		        /* value of term (weight, tf*idf usually)  */
}* Feature;

/* transformations on features 
 (weighting, selection and transformation) */
typedef enum {
  	/* Weighting strategies */
	TC_WEIGHT_TFIDF = 1 << 0,	/* tf*idf weighting */
	TC_WEIGHT_ENTROPY = 1 << 1,	/* weighting based on term entropy */
	TC_GLOBAL_WEIGHT_DOC = 1 << 2,	/* global weighting based on document */
	TC_GLOBAL_WEIGHT_CLASS = 1 << 3,	/* global weighting based on class */
	TC_COSINE_NORMALIZATION = 1 << 4,  /* normalization of feature weight by cosine */
	TC_WEIGHT_ICF = 1 << 5,         /* global weighting using inverse class frequency */

	/* Feature transformation */
	TC_LSI = 1 << 6		/* Latent semantic indexing (not implemented) */
} FeatureTransformations;	

Feature feature_new(int id, double w);
void feature_free(Feature *f);
void feature_dump(Feature f);
void feature_gfunc_free(gpointer data, gpointer user_data);

#endif /*__FEATURE_H_*/
