 /***************************************************************************
 *            methods.h
 *
 *  Tue Nov 11 17:36:08 2003
 *  Copyright  2003  Arturo Montejo Raez
 *  arturo.montejo.raez@cern.ch
 ****************************************************************************/
#ifndef _METHODS_H
#define _METHODS_H

#include <glib.h>
#include "bitset.h"
#include "method.h"

/* -------------------------------------------------------------------------- 
   DATA TYPES
-------------------------------------------------------------------------- */
typedef GPtrArray * Methods;

/* -------------------------------------------------------------------------- 
   FUNCTIONS
-------------------------------------------------------------------------- */

/* Methods functions */

Methods 
methods_new(void);

Methods 
methods_new_sized(guint size);

Methods
methods_new_clone(Methods m);

void 
methods_add_with_commands(Methods *methods, gchar *learn_command, gchar *classify_command);

void 
methods_add(Methods *methods, Method minf);

void methods_remove(Methods *m, guint index);

/* Returns true if there is any method based on an external classifier */
gboolean 
methods_any_command(Methods m);

/* Returns true if there is any method based on a built-in classifier */
gboolean 
methods_any_builtin(Methods m);

Method
methods_get(Methods methods, guint index);

void
methods_free(Methods *methods);

Method
methods_get_best(Methods ms, MethodSelection measure);

/* This Method must be destroyed after use */
Method
methods_macro_avg(Methods ms);

/* This Method must be destroyed after use */
Method
methods_micro_avg(Methods ms);

void
methods_serialize(Methods methods, gchar *file_path);

Methods 
methods_new_deserialize(gchar *file_path);

void
methods_dump(Methods m);

void
methods_evaluate_external_in_docs(Methods *docs_stats, BitSet class_docs, BitSet eval_docs, char * predictions_path);

void
methods_evaluate_builtin_in_docs(Methods *docs_stats, Samples ss, BitSet assignment);

void
methods_reset_measures(Methods *m);

#endif /* _METHODS_H */
