#ifndef __MODEL_H_
#define __MODEL_H_

#include "strint_table.h"
#include "inverted_index.h"
#include "pt_parser.h"
#include "sample.h"

/* ----------------------------------------------------------------------------
   Data type definitions
*/

typedef struct model_struct
{
  /* paths */
  gchar *datadir;		
  gchar *ptextdir;	
  gchar *classdir;	
  gchar *filepath_samples;
  gchar *filepath_samples_ascii;
  gchar *filepath_term_ids;
  gchar *filepath_class_ids;
  gchar *filepath_doc_ids;
  gchar *filepath_doc_info;
  gchar *filepath_term_doc_ii;
  gchar *filepath_term_doc_idx;
  gchar *filepath_class_doc_ii;
  gchar *filepath_class_doc_idx;
  gchar *filepath_pt_parser;
	
  StrIntTable term_id;
  StrIntTable doc_id;
  StrIntTable class_id;
  GArray *doc_info;	/* vector of DocInfo elements */
  InvertedIndex term_doc;
  InvertedIndex class_doc;
  FILE *fdocsv;		/* document vectors file */
  PTParser pt_parser;
} *Model;

extern struct params_struct params;

/* ----------------------------------------------------------------------------
   Macros and definitions
*/

/* Data file names */
#define MODEL_FILE_SAMPLES     "samples.vcs"
#define MODEL_FILE_SAMPLES_ASCII  "samples_ascii.vcs"
#define MODEL_FILE_TERM_IDS    "term_id.tt"
#define MODEL_FILE_CLASS_IDS   "class_id.tt"
#define MODEL_FILE_DOC_IDS     "doc_id.tt"
#define MODEL_FILE_DOC_INFO    "doc.inf"
#define MODEL_FILE_TERM_DOC_II "term_doc.ii"
#define MODEL_FILE_TERM_DOC_IDX "term_doc.idx"
#define MODEL_FILE_CLASS_DOC_II "class_doc.ii"
#define MODEL_FILE_CLASS_DOC_IDX "class_doc.idx"
#define MODEL_FILE_PT_PARSER "pt_parser.inf"

/* ----------------------------------------------------------------------------
   Functions declarations
*/
/* 	Creates a new model for plain text documents found in 'docdir' labeled with 
	classes found in 'classdir' and stores the model in the 'datadir' directory.
	This function just sets the parameters of the model, but does not build it.
	To do so call model_build afterwards.
*/
Model model_new (char *datadir);

/* Opens an existing model */
Model model_new_read (char *datadir);

/* Generates the model computing all the needed statistics and inverted indexes
 */
void
model_build (Model * model, char *ptextdir, char *classdir, PTParserParams * pt,
	     guint minfreq);

/* Takes a file and generates a sample using model information and the given parser */
Sample
model_plain_text_process (Model * model, gchar *file, guint minfreq);

/* Outputs a set of classes in their string form */
void model_show_classes_strings (Model m, Sample classes);

/* Liberates all the resources holded by the model */
void model_free (Model * model);



#endif /*__MODEL_H_*/
