/***************************************************************************
 *            pt_parser.h
 *
 *  Thu Nov 27 11:43:16 2003
 *  Copyright  2003  Arturo Montejo Raez
 *  arturo.montejo.raez@cern.ch
 ****************************************************************************/

/*
	Plain text processing tools
*/

#ifndef __PTPROC_H_
#define __PTPROC_H_

#include <glib.h>
#include <stdio.h>
#include "stemmer.h"
#include "stopper.h"

/* ---------------------------------------------------------------------------
	FLAGS for plain text processing 
*/
typedef enum {
	TC_LOWER_CASE   = 1 << 0,
	TC_STOPPER 		= 1 << 1,
	TC_MULTIWORDS	= 1 << 2,
	TC_STEMMER 		= 1 << 3
} PTParserFlags;

/* Operations performed by default */
#define TC_PTPROC_DEFAULT		TC_LOWER_CASE | TC_STOPPER | TC_STEMMER

/* ---------------------------------------------------------------------------
	Data type definitions 
*/

/* Plain text processing parameters */
typedef struct {
	PTParserFlags flags;         /* flags to control processing behaviour (see above) */
	guint maxlen;                /* maximal length of a term allowed */
	guint minlen;				 /* minimal length of a term allowed */
	char *stopwdlist;			 /* file containing stop words to be removed */
} PTParserParams;

/* The plain text parser */
typedef struct pt_parser_struct {
	PTParserParams params;
	Stemmer stemmer;        /* Porter stemmer object */
	Stopper stopper;        /* stop words removal object */	
} * PTParser;

/* ---------------------------------------------------------------------------
	Function definitions
*/

PTParser pt_parser_new(PTParserParams *params);
void pt_parser_free(PTParser *p);
void pt_parser_save(PTParser p, gchar *filepath);
PTParser pt_parser_new_read(gchar *filepath);

/* This function reads the next processed term in the file 
   strbuff -- pointer to a char buffer to store the read term 
   RETURNS: 0 if end of file reached, 1 otherwise 
*/
int pt_parser_read (PTParser p, FILE * fin, char *strbuff);

#endif /*__PTPROC_H_*/
