#ifndef __SAMPLE_H_
#define __SAMPLE_H_

#include <glib.h>
#include <stdio.h>
#include "feature.h"
#include "term_info.h"

/* ---------------------------------------------------------------------
   Sample type
*/

#define ERROR_INVALID_SAMPLE 1

typedef struct sample_struct
{
  tc_id id;             /* id for the document */
  double y;		/* class: +1/-1 */
  double modulus;	/* norm of elements in the list */
  double max_w;		/* maximum weight of features in the sample */
  double min_w;         /* minimum weight of features in the sample */
  tc_id max_id;		/* maximum index of features in the sample */
  guint total;          /* total number of features in the sample */ 
  GList *features;	/* list of features with weights */
  GList *it;		/* iterator pointer */
} *Sample;

/* ------------------------------------------------------------------------
   Sample functions
*/

Sample sample_new ();

Sample 
sample_new_from_strint_table(StrIntTable tt, StrIntTable *termtt);

void sample_free (Sample * s);

/* frees the memory allocated, it also frees features space */
void
sample_gfunc_free (gpointer data, gpointer user_data);

/* Sets the iterator pointer to the first element in the list */
void sample_it_reset (Sample * s);

/* Returns the actual term *pointed* (no new space is allocated)
   by the internal iterator and moves it forward */
Feature sample_it_next (Sample * s);

/* Returns TRUE if internal iterator has reached last element in the
   list, FALSE otherwise */
gboolean sample_it_end (Sample s);

/* Removes the feature returned in the last sample_it_next() */
void sample_it_remove(Sample *s);

void sample_dump (Sample s);

/* Writes the sample in a file 
   When written in non binary format features are sorted by id
*/
void sample_serialize (Sample s, FILE * fhnd, gboolean binary);

/* Reads the sample allocated at the given offset */
Sample sample_new_deserialize (FILE * f, long offset, gboolean binary);

/* Reweights features in the sample */
Sample sample_new_reweight (Sample s, TermInfoArray term_info, BitSet train_docs, FeatureTransformations ftrans);

/* Sorts the sample features by id 
   NOTE: it changes the iterator position. If the iterator was at the begining, 
   after calling this function it may not be at the begining, so to reset it use 
   sample_it_reset
*/
void sample_id_sort(Sample *s);

/* Adds another feature to the sample 
   Once the last feature has been added, sample_add_feature_finished should
   be called */
void sample_add_feature(Sample *s, Feature f);

/* Return a bitset (to be deallocated by the caller) composed by the ids in the sample */
BitSet sample_to_bitset(Sample s);

/* This computes the module of the sample, only useful when used after a series
   of sample_add_feature calls */
void sample_add_feature_finished(Sample *s);

#endif /*__SAMPLE_H_*/
