#ifndef __SAMPLES_H_
#define __SAMPLES_H_

#include <glib.h>
#include "sample.h"

/* ---------------------------------------------------------------------
   Samples type
 */

typedef struct samples_struct {
  GList *samples;             /* list of term with weights */
  GList *it;                  /* iterator pointer */
  guint max_id;                /* max document id in the collection */
  guint max_feature_id;        /* maximum term id in the collection */
  double max_w;                /* maximum weight found in samples features */
  double min_w;                /* minimum weight found in samples features */
  double r2;                   /* square of maximum modulus in the collection */
  guint total;                 /* total number of samples in the collection */
}* Samples;

Samples samples_new();

void samples_free(Samples *s);

/* This function creates a shallow copy of the list, i.e. list is
   copied, but not elements. The returned samples MUST be freezed with
   samples_shallow_free() */
Samples 
samples_shallow_clone(Samples ss);

/* This function freezes samples struct and the list, but not each
   sample. Use this function only for samples created using the
   samples_shallow_clone function. */
void 
samples_shallow_free(Samples *ss);

void samples_add(Samples *ss, Sample s);

/* To be invoqued once all the samples_add calls has been performed */
void samples_add_finished(Samples *ss);

/* It sets the label of samples and remove those ones that are not
   neither positive nor negative samples */
void samples_assign(Samples *ss, BitSet posdocs, BitSet negdocs);

/* Reads a file of samples */
Samples samples_new_read (gchar *samples_path, BitSet docs);

/* Creates a collections of samples composed by classes instead of terms */
Samples samples_new_for_class_filter(guint nclasses, InvertedIndex class_doc, BitSet doc_ids);

/* Sorts the list of samples by their distance to the given sample */
void samples_sort_by_similarity(Samples *ss, Sample s);

/* Reads a file of samples, if ids are not NULL samples will get those id numbers. */
int samples_read(Samples *ss, char * file, BitSet ids);

/* Sets the iterator pointer to the first sample in the list */
void samples_it_reset(Samples *ss);

/* Returns the actual sample pointed by the internal iterator and moves
   it forward */
Sample samples_it_next(Samples *ss);

/* Returns TRUE if internal iterator has reached last element in the
   list, FALSE otherwise */
gboolean samples_it_end(Samples ss);

/* Removes the node before the last returned element of a call to samples_it_next() */
void samples_it_remove(Samples *ss);

void samples_dump(Samples ss);

#endif /*__SAMPLE_H_*/
