#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "strint_table.h"
#include "tc_utils.h"


#define STRINT_STR_BUFFER_LENGTH 255

/* -------------------------------------------------------------------------- */
/* Public functions */
/* -------------------------------------------------------------------------- */
StrIntTable
strint_table_new (void)
{
  StrIntTable tt = g_new0 (struct strint_table_struct, 1);
  tt->table =
    (GHashTable *) g_hash_table_new_full (g_str_hash, g_str_equal,
					  g_free, g_free);
  tt->max_id = -1;

  return tt;
}

/* -------------------------------------------------------------------------- */
gboolean
__always_remove (gpointer key, gpointer value, gpointer user_data)
{
  return TRUE;
}

void
strint_table_free (StrIntTable * tt)
{
  if (!(*tt))
    tc_error ("strint_table_free: null pointer\n");

  //if (kk) strint_table_dump(*tt);       
  g_hash_table_destroy ((*tt)->table);
  g_free (*tt);
}

/* -------------------------------------------------------------------------- */
/* 	Retrieves the integer associated to the given string.
	If no key found -1 is returned  */
tc_id
strint_table_get (StrIntTable tt, gchar * str)
{
  tc_id *p = NULL;

  if (!tt)
    tc_error ("strint_table_get: null pointer\n");
  p = (tc_id *) g_hash_table_lookup (tt->table, str);
  if (p != NULL)
    return *p;
  else
    return -1;
}

/* -------------------------------------------------------------------------- */
void
strint_table_set (StrIntTable * tt, gchar * str, tc_id id)
{
  tc_id *new_int;

  if (!*tt)
    tc_error ("strint_table_set: null pointer\n");
  new_int = g_new (tc_id, 1);
  *new_int = id;
  g_hash_table_insert ((*tt)->table, g_strdup (str), new_int);
  if ((*tt)->max_id < id)
    (*tt)->max_id = id;
}

/* -------------------------------------------------------------------------- */
void
__strint_table_dump_item (gpointer key, gpointer value, gpointer user_data)
{
  gchar *str = (gchar *) key;
  tc_id *id = (tc_id *) value;
  printf ("%s -> %d\n", str, *id);
}

/* -------------------------------------------------------------------------- */
void
strint_table_dump (StrIntTable tt)
{
  g_hash_table_foreach (tt->table, __strint_table_dump_item, NULL);
  printf ("max_id: %d\n", tt->max_id);
}

/* -------------------------------------------------------------------------- */
/* private function to be used by strint_table_filter_value */
gboolean
__strint_table_is_value (gpointer key, gpointer value, gpointer user_data)
{
  return (*(tc_id *) user_data > 0
	  && *(tc_id *) value < *(tc_id *) user_data);
}

/* -------------------------------------------------------------------------- */
/* This function removes those entries with value less than 'value' (0 do not filter)*/
void
strint_table_filter_value (StrIntTable * tt, tc_id value)
{
  g_hash_table_foreach_remove ((*tt)->table, __strint_table_is_value,
			       &value);
}

/* -------------------------------------------------------------------------- */
StrIntTable
strint_table_new_read (char *file)
{
  StrIntTable tt = strint_table_new ();
  char str[STRINT_STR_BUFFER_LENGTH];
  guint strl;
  tc_id id;
  FILE *f = fopen (file, "r");
  if (!f)
    tc_error ("strint_table_read: unable to open file\n");
  while (!feof (f))
    {
      fread (&strl, sizeof (guint), 1, f);	/* string length */
      fread (&str, strl + 1, 1, f);	/* string */
      fread (&id, sizeof (tc_id), 1, f);	/* integer id */
      strint_table_set (&tt, str, id);
    }
  fclose (f);
  return tt;
}

/* -------------------------------------------------------------------------- */
void
__strint_table_save_item (gpointer key, gpointer value, gpointer user_data)
{
  guint strl = strlen (key);
  fwrite (&strl, sizeof (guint), 1, (FILE *) user_data);	/* string length */
  fwrite (key, strl + 1, 1, (FILE *) user_data);	/* string */
  fwrite (value, sizeof (tc_id), 1, (FILE *) user_data);	/* integer id */	
}

void
strint_table_save (StrIntTable tt, char *file)
{
  FILE *f = fopen (file, "w");
  if (!f)
    tc_error ("strint_table_save: unable to create file\n");
  g_hash_table_foreach (tt->table, __strint_table_save_item, f);
  fclose (f);
}
