/***************************************************************************
 *            tc_utils.c
 *
 *  Mon Oct 27 14:33:44 2003
 *  Copyright  2003  Arturo Montejo Raez
 *  arturo.montejo.raez@cern.ch
 ****************************************************************************/

#include <stdlib.h>
#include <ctype.h>
#include "tc_utils.h"
#include "tc_types.h"

extern gboolean verbose;

/* -------------------------------------------------------------------------- */
/* This function replaces fscanf in order to control string buffer over range */
inline gboolean
tc_fscanstr (FILE * f, char *strbuff)
{
  char c;
  int len = 0;

  c = getc (f);
  while (!isspace ((int) c) && isprint ((int) c) && c != EOF
	 && len < TC_BUFFER_LENGTH - 1)
    {
      strbuff[len++] = c;
      c = getc (f);
    }

  strbuff[len] = 0;	/* 0 terminated string */
  return (c != EOF);
}

/* -------------------------------------------------------------------------- */
/* Same as above, but for a whole line */
inline gboolean
tc_fscanline (FILE * f, char *strbuff)
{
  char c;
  int len = 0;

  c = getc (f);
  while (c != '\n' && isprint ((int) c) && c != EOF && len < TC_BUFFER_LENGTH - 1)
    {
      strbuff[len++] = c;
      c = getc (f);
    }
  if (len == TC_BUFFER_LENGTH - 1)
    tc_error("tc_fscanline: string too long");
  strbuff[len] = 0;	/* 0 terminated string */
  return (c != EOF);
}

/* -------------------------------------------------------------------------- */
void
tc_error (char *msg)
{
  fprintf (stderr, "%s\n", msg);
  exit (EXIT_FAILURE);
}

/* -------------------------------------------------------------------------- */
void
tc_error_str (char *msg, char *str)
{
  fprintf (stderr, msg, str);
  fprintf (stderr, "\n");
  exit (EXIT_FAILURE);
}

/* -------------------------------------------------------------------------- */
void
tc_message (char *msg)
{
  if (!verbose)
    return;

  printf (msg);
  fflush (stdout);
}

/* -------------------------------------------------------------------------- */
void
tc_message_int (char *msg, int i)
{
  if (!verbose)
    return;

  printf (msg, i);
  fflush (stdout);
}

/* -------------------------------------------------------------------------- */
void
tc_message_str (char *msg, char *str)
{
  if (!verbose)
    return;

  printf (msg, str);
  fflush (stdout);
}
