/***************************************************************************
 *            term_info.h
 *
 *  Thu Nov 27 11:43:16 2003
 *  Copyright  2003  Arturo Montejo Raez
 *  arturo.montejo.raez@cern.ch
 ****************************************************************************/

#ifndef _TERM_INFO_H
#define _TERM_INFO_H

#include <glib.h>
#include "strint_table.h"
#include "intstr_table.h"
#include "inverted_index.h"
#include "bitset.h"
#include "doc_info.h"


/* ---------------------------------------------------------------------------
   Data definitions and macros
   --------------------------------------- */

/*  This structure stores global information about each term  */
typedef struct term_info_struct
{
	tc_id id;       /* id of the term */
	tc_id new_id;	/* id after reduction (-1 means discarded feature) */
	guint df;		/* number of documents the term appears */
	guint cf;		/* number of classes the term is related to */
	guint accum_freq; /* accumulated frequency of the term across the collection (F_i) */
	double ig;		/* information gain */
	double entropy; /* average uncertainty or entropy for the feature at class level (G_i) */
	double idf;		/* inverse document frequency (G_i) */
	double icf;		/* inverse class frequency (G_i) */
	double tmp_w;   /* temporal weight (for cosine normalization) */
} TermInfo;

typedef GArray *TermInfoArray;

/* ---------------------------------------------------------------------------
   Functions
   --------------------------------------- */

/* Compute document frequency, information gain, class frequency, 
   inverse document frequency and inverse class frequency for each term 
*/
TermInfoArray
term_info_new_compute (InvertedIndex term_doc, InvertedIndex class_doc, BitSet train_docs, guint fig_size);

/* This function sets to -1 those features which should be discarded 
  It filters by information gain (if fig_size > 0) and/or document frequency (if df_min > 0)
*/
void
term_info_reduce(TermInfoArray *term_info, guint fig_size, guint df_min);

/* Dumps new_id of terms */
void
term_info_dump_new_id(TermInfoArray term_info, IntStrTable idterm);

/* Returns a pointer to term information */
TermInfo *term_info_get(TermInfoArray term_info, guint index);

/* This function computes the entropy of every term */
void
term_info_compute_entropy(FILE * fdocsv, TermInfoArray* term_info, DocInfoArray doc_info, BitSet docs);

/* Saves array to disk */
void
term_info_save(TermInfoArray term_info, gchar *filepath);

/* Reads array from disk */
TermInfoArray
term_info_new_read(gchar *filepath);

void term_info_free(TermInfoArray *term_info);

#endif /* _TERM_INFO_H */
